// $Id: albert.h,v 1.15 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#ifndef _ALBERT_H_
#define _ALBERT_H_




#include "sprite.h"
#include "enums.h"
#include "constants.h"
#include "log.h"
#include "crystal.h"
#include "SDL.h"
#include "fish.h"
#include "collisions.h"
#include "starparticlesystem.h"
#include "userinput.hh"
#include "gfxstore.hh"
#include "blitter.hh"




extern FS::UserInput * my_input;
extern bool escape_pressed;
extern FS::GfxStore * my_gfx;
extern FS::Blitter * my_blitter;




namespace FS
{

// FIXME: ?!
static const int NUM_COLL_BOXES = 4;
static const int COLL_BOX_X_OFFSETS[] = { 34, 31, 38, 28 };
static const int COLL_BOX_Y_OFFSETS[] = { 28, 34, 31, 38 };
static const int COLL_BOX_WIDTHS[] = { 12, 25, 12, 25 };
static const int COLL_BOX_HEIGHTS[] = { 25, 12, 25, 12 };
    
class Albert : public Sprite
{
    private:
            
        Orientation my_orientation;
    
        int actions;
        
        bool airborne;
          
        int start_time; 
        int start_x;
        int start_y;

        int current_frame;
        //ImageIDs current_strip;
        GfxStore::Image_Name current_strip;
        
        Log * host_log;
        
        int coll_box_x_offsets[NUM_COLL_BOXES];
        int coll_box_y_offsets[NUM_COLL_BOXES];
        SDL_Rect my_coll_boxes[NUM_COLL_BOXES];
        
        void process_keypress(int t);
        void process_animation(int t);
        
        void calc_rects();
        void calc_strip();
        void check_host_log();
        
        //SDL_Rect image_section_rect;
        
        void turn_on_action(int a) { actions |= a; }
        void turn_off_action(int a) { actions &= ~a; }
        int test_action(int a) const { return (actions & a); }
        
        // Albert landed in the water/got zapped and splashing/zapping 
        // animation is over.
        bool splashed_zapped_out;
        
        bool left;
        
        bool next_jump_double;
        
        StarParticleSystem stars;
        
    public:
        Albert();
        ~Albert();
        
        void reset();
        void update(int t, bool is_running);
        
        int upper_y() const { return y + coll_box_y_offsets[my_orientation]; }
        int lower_y() const { return y + coll_box_y_offsets[my_orientation] + 
                my_coll_boxes[my_orientation].h; }
        
        bool is_airborne() const { return airborne; }
        bool is_on_log() const { return (test_action(FLOATING) > 0); }
        int is_in_water_or_being_zapped() const; 
        
        void set_new_host_log(Log * l);
        
        // If Albert is on a log, returns that log's row (see RowDescription enum
        // in "enums.h"); else returns -1 (an invalid row).
        int get_host_log_row() const; // { return host_log->get_row(); }
        const Log* get_host_log() const { return host_log; }
        
        void start_splash(int t);
        void start_zap(int t);
        
        void start_leave_with_fish(int t);
        void start_leave_in_a_huff(int t);
        
        bool is_splashed_zapped_out() const { return splashed_zapped_out; }
        
        bool has_left() const { return left; }
        
        // Redefinition of Sprite::get_coll_box(). No dynamic binding.
        const SDL_Rect & get_coll_box();
        
        // Redefinition of Sprite::get_bounding_box(). This is so we can make
        // Albert's bounding box a little bigger than image size.
        const SDL_Rect & get_bounding_box();
        
        void set_next_jump_double() { next_jump_double = true; }
        
	bool has_stars() const { return stars.is_active(); }
        
        void schedule_draw( /*PlayDisplay * pd*/ );
	
}; // class Albert

} // namespace FS




#endif
