// $Id: blitter.cc,v 1.7 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "blitter.hh"




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::Blitter::Blitter()
{
    reset_indices();

} // FS::Blitter::Blitter()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::Blitter::~Blitter()
{
    // nothing

} // FS::Blitter::~Blitter()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::Blitter::request_blit_image( GfxStore::Image_Name _id, const SDL_Rect & _dest )
{
    blit_requests[blit_requests_index].type = IMAGE;   
    blit_requests[blit_requests_index].img_blit.id = _id;
    blit_requests[blit_requests_index].img_blit.dest = _dest;
    blit_requests[blit_requests_index].img_blit.copy_entire_surface = true;
    ++blit_requests_index;
    
} // FS::Blitter::request_blit_image()

// **************************************************

void FS::Blitter::request_blit_image( GfxStore::Image_Name _id, const SDL_Rect & _src,
        const SDL_Rect & _dest )
{
    blit_requests[blit_requests_index].type = IMAGE;     
    blit_requests[blit_requests_index].img_blit.id = _id;
    blit_requests[blit_requests_index].img_blit.src = _src;
    blit_requests[blit_requests_index].img_blit.dest = _dest;
    blit_requests[blit_requests_index].img_blit.copy_entire_surface = false;
    ++blit_requests_index;
    
} // FS::Blitter::request_blit_image

// **************************************************

void FS::Blitter::request_blit_text( GfxStore::Font_Name _id, const SDL_Rect & _dest, 
        const char * _text, bool _center )
{
    blit_requests[blit_requests_index].type = TEXT;
    blit_requests[blit_requests_index].txt_blit.id = _id;
    blit_requests[blit_requests_index].txt_blit.dest = _dest;
    blit_requests[blit_requests_index].txt_blit.text = _text;
    blit_requests[blit_requests_index].txt_blit.center = _center;
    ++blit_requests_index;
     
} // FS::Blitter::request_blit_text()

// **************************************************


bool FS::Blitter::request_erase( GfxStore::Image_Name _id, const SDL_Rect & _r )
{
    erase_requests[erase_requests_index].id = _id;
    erase_requests[erase_requests_index].r = _r;
    ++erase_requests_index;
    return true;
    
} // FS::Blitter::request_erase()

// **************************************************

bool FS::Blitter::request_update(const SDL_Rect & r)
{
    update_requests[update_requests_index] = r;
    ++update_requests_index;
    return true;
    
} // FS::Blitter::request_update()

// **************************************************

void FS::Blitter::blit_and_update( GfxStore::Image_Name id )
{
    bool update_whole_screen = false;
    
    if ( id != GfxStore::NONE )
    {
        SDL_BlitSurface( my_gfx->get_image(id), NULL, screen, NULL );
        update_whole_screen = true;
    }
    else
    {
        for ( int i = 0; i < erase_requests_index; ++i )
        {
            SDL_BlitSurface( my_gfx->get_image( erase_requests[i].id ),
                             &erase_requests[i].r,
                             screen,
                             &erase_requests[i].r );
        } // for
    } // if ... else
    
    for ( int i = 0; i < blit_requests_index; ++i )
    {
        switch ( blit_requests[i].type )
        {
            case IMAGE:
                if ( blit_requests[i].img_blit.copy_entire_surface )
                {
                    SDL_BlitSurface( 
                        my_gfx->get_image( blit_requests[i].img_blit.id ),
                        NULL,
                        screen,
                        &blit_requests[i].img_blit.dest );
                }
                else
                {
                    SDL_BlitSurface( 
                        my_gfx->get_image( blit_requests[i].img_blit.id ),
                        &blit_requests[i].img_blit.src,
                        screen,
                        &blit_requests[i].img_blit.dest );
                } // if ... else    
                break;
            case TEXT:
                if ( blit_requests[i].txt_blit.center )
                {
                    (my_gfx->get_font( blit_requests[i].txt_blit.id )).writeCenter(
                            screen, 
                            blit_requests[i].txt_blit.text,
                            blit_requests[i].txt_blit.dest.y );
                }
                else
                {
                    (my_gfx->get_font( blit_requests[i].txt_blit.id )).write(
                            screen, 
                            blit_requests[i].txt_blit.text,
                            blit_requests[i].txt_blit.dest.x,
                            blit_requests[i].txt_blit.dest.y ); 
                } // if ... else   
                break;
        } // switch
    } // for
            
    // update
    if ( update_whole_screen )
    {
        SDL_UpdateRect( screen, 0, 0, 0, 0 );
    }
    else
    {
        SDL_UpdateRects( screen, update_requests_index, update_requests );
    } // if ... else

    reset_indices();
    
} // FS::Blitter::blit_and_update()

// **************************************************

void FS::Blitter::reset_indices()
{
    blit_requests_index = 0;
    erase_requests_index = 0;
    update_requests_index = 0;
    
} // FS::Blitter::reset_indices()

// **************************************************
