// $Id: blitter.hh,v 1.5 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#ifndef _BLITTER_HH_
#define _BLITTER_HH_




#include "SDL.h"
#include "gfxstore.hh"




extern SDL_Surface * screen;
extern FS::GfxStore * my_gfx;




namespace FS
{

class Blitter
{

    public:
            
        enum Blit_Request_Type { IMAGE, TEXT };
        static const int MAX_RECTS = 150;   
        
    private:

        struct Blit_Image_Request
        {
            GfxStore::Image_Name id;
            SDL_Rect src;
            SDL_Rect dest;
            bool copy_entire_surface;
        }; // struct Blit_Image_Request
        
        struct Blit_Text_Request
        {
            GfxStore::Font_Name id;
            SDL_Rect dest;
            const char * text;
            bool center;
        }; // struct Blit_Text_Request
        
        struct Blit_Request 
        {
            Blit_Request_Type type;
            union 
            {
                Blit_Image_Request img_blit;
                Blit_Text_Request txt_blit;
            }; // union 
        }; // struct Blit_Request
        
        struct Erase_Request
        {
            GfxStore::Image_Name id;
            SDL_Rect r;
        }; // struct Erase_Request
        
        Blit_Request blit_requests[MAX_RECTS];
        Erase_Request erase_requests[MAX_RECTS];
        SDL_Rect update_requests[MAX_RECTS];
       
        int blit_requests_index;
        int erase_requests_index;
        int update_requests_index;
        
        void reset_indices();
        void test_this();
        
    public:

        Blitter();
        ~Blitter();
        
        // An overloaded version of request_blit_image allows you to specify 
        // a source rectangle for the image to be blitted; otherwise the whole
        // image is used.
        void request_blit_image( GfxStore::Image_Name _id, const SDL_Rect & _dest );
        void request_blit_image( GfxStore::Image_Name _id, const SDL_Rect & _src,
                const SDL_Rect & _dest );
        void request_blit_text( GfxStore::Font_Name _id, const SDL_Rect & _dest, 
                const char * _text, bool _center = false );
        bool request_erase( GfxStore::Image_Name _id, const SDL_Rect & _r );
        bool request_update(const SDL_Rect & r);
        
        // If 'id' is anything other than GfxStore::NONE, the given image will
        // be blitted first (at 0,0), all scheduled erase and update requests 
        // will be ignored, and the whole screen will be updated. Can use this 
        // to clear the screen when, e.g. starting a new level of a game.
        void blit_and_update( GfxStore::Image_Name id = GfxStore::NONE );
        
}; // class Blitter

} // namespace FS




#endif
