// $Id: crystal.cc,v 1.7 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "crystal.h"




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::Crystal::Crystal( int _x, int _y, int _my_row, CrystalColor color )
{
    x = old_x = _x + CRYSTAL_X_OFFSET;
    y = old_y = _y + CRYSTAL_Y_OFFSET;
    width = CRYSTAL_WIDTH;
    height = CRYSTAL_HEIGHT;
    
    my_row = _my_row;
    original_color = current_color = color;
    
    my_id = CRYSTAL;
    
    sprite_dest_rect.y = y;
    
    erase_rect.y = y;
    erase_rect.w = width;
    erase_rect.h = height;
    
    update_rect.y = y;
    update_rect.h = height;
    
    coll_box_x_offset = CRYSTAL_COLL_BOX_X_OFFSET;
    coll_box_y_offset = CRYSTAL_COLL_BOX_Y_OFFSET;
    coll_box.w = CRYSTAL_COLL_BOX_WIDTH;
    coll_box.h = CRYSTAL_COLL_BOX_HEIGHT;

    must_deactivate = false;
    active = true;
        
} // FS::Crystal::Crystal
    
        
// ******************
// *** DESTRUCTOR ***
// ******************
FS::Crystal::~Crystal()
{
    // ?!
    
} // FS::Crystal::~Crystal()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::Crystal::reset()
{
    current_color = original_color;
    active = true;
    must_deactivate = false;
    
} // FS::Crystal::reset()

// **************************************************
        
void FS::Crystal::update( const ScrollingData & sd )
{
    // Need to scroll whether crystal's active or not - it will
    // be reactivated if the level is restarted.
    scroll_x(sd);   // i.e. MapSprite::scroll_x
    
    if ( active )
    {
        if ( calc_rects(sd, my_row) )   // see CollectableSprite::calc_rects
        {
            if ( !must_deactivate )
            {
                my_blitter->request_blit_image( 
                        (GfxStore::Image_Name) (current_color + 13),   // FIXME!!!
                        sprite_dest_rect );
            }
            else
            {
                must_deactivate = false;
                deactivate();   // see Sprite::deactivate
            } // if ... else
        } // if
    } // if
    
} // FS::Crystal::update
                    
// **************************************************
