// $Id: getreadydisplay.cc,v 1.13 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "getreadydisplay.h"
#include "SDL_image.h"
#include <cstdio>
#include <iostream>
#include <string>




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::GetReadyDisplay::GetReadyDisplay(/*SDL_Surface * s*/)
{
    //screen = s;
    
    dest_rect.x = 355;
    dest_rect.y = 90;
    erase_rect.x = 355;
    erase_rect.y = 90;
    erase_rect.w = 90;
    erase_rect.h = 50;
    
} // FS::GetReadyDisplay::GetReadyDisplay()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::GetReadyDisplay::~GetReadyDisplay()
{
    // nothing
    
} // FS::GetReadyDisplay::~GetReadyDisplay() 




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::GetReadyDisplay::start_draw(int gt, int lev)
{
    start_time = gt;
    
    std::string message;
    char digits[4]; // allows for 999 levels!
    
    sprintf(digits, "%d", lev);
    message = "About to start level ";
    message += digits;
    message += " . . .";
    
    SDL_Rect my_rect;
    
    my_rect.y = 50;
    my_blitter->request_blit_text( GfxStore::SMALL_FONT, my_rect, message.c_str(), true );
    my_rect.y = 90;
    my_blitter->request_blit_text( GfxStore::SMALL_FONT, my_rect, "Get Ready!", true ); 
    
    // FIXME: second arg shouldn't be hard-coded.
    my_rect.y = 130;
    my_blitter->request_blit_text( GfxStore::SMALL_FONT, my_rect, "10", true ); 
    
    my_blitter->blit_and_update( GfxStore::GET_READY );
    
} // FS::GetReadyDisplay::start_draw()
     
// **************************************************

bool FS::GetReadyDisplay::redraw(int gt)
{
    int seconds_passed = (gt - start_time) / 1000;
    bool jump_pressed = false;
    UserInput::Keyboard_Event ke;
    
    while ( my_input->next_event(ke) )
    {
        if ( ke == UserInput::JUMP_KEY_PRESSED )
        {
            jump_pressed = true;
            break;
        } // if
    } // while
    
    if ( (seconds_passed == SECONDS_TO_WAIT) || jump_pressed )
    {
        //finished = true;
        //return;
        return true;
    } // if
    
    SDL_Rect my_rect = { 390, 130, 50, 50 };
    char seconds[3];
    sprintf(seconds, "%d", (SECONDS_TO_WAIT - seconds_passed));
    my_blitter->request_erase( GfxStore::GET_READY, my_rect );
    my_blitter->request_blit_text( GfxStore::SMALL_FONT, my_rect, seconds, true );
    my_blitter->request_update( my_rect );
    my_blitter->blit_and_update();
    
    return false;
    
} // FS::GetReadyDisplay::redraw()

// **************************************************

void FS::GetReadyDisplay::start_draw_level_select(int furthest_lev)
{
    current_start_level = furthest_lev;
    
    char lev_string[4];
    sprintf(lev_string, "%d", current_start_level);
    
    SDL_Rect my_rect;
    my_rect.y = 50;
    my_blitter->request_blit_text( GfxStore::SMALL_FONT, my_rect, 
            "Please select start level . . .", true );
    my_rect.y = 90;
    my_blitter->request_blit_text( GfxStore::SMALL_FONT, my_rect, lev_string, true ); 
    
    my_blitter->request_blit_image( GfxStore::UPPER_LIMIT, dest_rect );
    
    my_blitter->blit_and_update( GfxStore::GET_READY );
    
} // FS::GetReadyDisplay::start_draw_level_select()
   
// **************************************************

int FS::GetReadyDisplay::redraw_level_select(int furthest_lev)
{
    bool changed = false;
    int selection = 0;
    UserInput::Keyboard_Event ke;
    
    while ( my_input->next_event(ke) )
    {
        if ( ke == UserInput::JUMP_KEY_PRESSED )
        {
            selection = current_start_level;
            break;
        }
        else if ( ke == UserInput::LEFT_KEY_PRESSED )
        {
            if ( current_start_level > 1 )
            {
                --current_start_level;
                changed = true;
            } // if
        }
        else if ( ke == UserInput::RIGHT_KEY_PRESSED )
        {
            if ( current_start_level < furthest_lev )
            {
                ++current_start_level;
                changed = true;
            } // if
        } // if ... else
    } // while
    
    if (changed)
    {
        char lev_string[4];
        sprintf(lev_string, "%d", current_start_level);
        
        my_blitter->request_erase( GfxStore::GET_READY, erase_rect );
        
        GfxStore::Image_Name limit;
        if ( current_start_level == 1 ) 
        { 
            limit = GfxStore::LOWER_LIMIT; 
        }
        else if ( current_start_level == furthest_lev ) 
        { 
            limit = GfxStore::UPPER_LIMIT; 
        }
        else 
        { 
            limit = GfxStore::MIDDLE; 
        } // if ... else
        
        my_blitter->request_blit_image( limit, dest_rect );
        
        SDL_Rect my_rect;
        my_rect.y = 90;
        my_blitter->request_blit_text( GfxStore::SMALL_FONT, my_rect, lev_string, true );
        
        my_blitter->request_update( erase_rect );
        my_blitter->blit_and_update();
    } // if
    
    return selection;
    
} // FS::GetReadyDisplay::redraw_level_select()
    
// **************************************************
// **************************************************
