// $Id: gfxstore.cc,v 1.4 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "gfxstore.hh"




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::GfxStore::GfxStore()
{
    // load gfx
    std::string filename;
    char digits[3]; // i.e. 2 digits and terminating null
    
    for ( int i = 0; i < NUM_IMAGES; ++i )
    {
        filename = (PKG_DATA_DIR "/images/fs_image_");
        sprintf(digits, "%02d", i);
        (filename += digits) += ".png";
        
        my_images[i] = SDL_DisplayFormatAlpha(IMG_Load(filename.c_str()));
        SDL_SetAlpha( my_images[i], SDL_SRCALPHA|SDL_RLEACCEL, 0 );
    } // for
    
    for ( int i = 0; i < NUM_FONTS; ++i )
    {
        filename = (PKG_DATA_DIR "/images/fs_font_");
        sprintf(digits, "%02d", i);
        (filename += digits) += ".png";
        
        my_font_images[i] = SDL_DisplayFormatAlpha(IMG_Load(filename.c_str()));
        SDL_SetAlpha( my_font_images[i], SDL_SRCALPHA|SDL_RLEACCEL, 0 );
        my_fonts[i].init( my_font_images[i] );  
    } // for
            
} // FS::GfxStore::GfxStore()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::GfxStore::~GfxStore()
{
    // free gfx
    for ( int i = 0; i < NUM_IMAGES; ++i )
    {
        SDL_FreeSurface(my_images[i]);
    } // for
    
} // FS::GfxStore::~GfxStore() 




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

SDL_Surface* FS::GfxStore::get_image(Image_Name n)
{
    return my_images[n];
    
} // FS::GfxStore::get_image

// **************************************************

const SFont& FS::GfxStore::get_font(Font_Name n) const
{
    return my_fonts[n];
    
} // FS::GfxStore::get_font
        
// **************************************************
// **************************************************
// **************************************************
// **************************************************
// **************************************************

