// $Id: gfxstore.hh,v 1.4 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#ifndef _GFXSTORE_HH_
#define _GFXSTORE_HH_




#include <SDL/SDL.h>
#include "Font.hpp"
#include <string>
#include <SDL/SDL_image.h>
#include <iostream>




namespace FS
{

class GfxStore
{
    public:
        
        static const int NUM_IMAGES = 56;
        // NONE is a dummy image name not used in any subsequent operations.
        enum Image_Name { 
                NONE = -1,
                GET_READY,
                LOWER_LIMIT,
                MIDDLE,
                UPPER_LIMIT,
                CATS_WHISKERS,
                BACKGROUND,
                LOG_RED,
                LOG_YELLOW,
                LOG_BLUE,
                LOG_ORANGE,
                LOG_GREEN,
                LOG_PURPLE,
                LOG_BROWN,
                CRYSTAL_RED,
                CRYSTAL_YELLOW,
                CRYSTAL_BLUE,
                CRYSTAL_WHITE,
                WALKING_N,
                WALKING_E,
                WALKING_S,
                WALKING_W,
                JUMPING_N,
                JUMPING_E,
                JUMPING_S,
                JUMPING_W,
                SPLASHING_N,
                SPLASHING_E,
                SPLASHING_S,
                SPLASHING_W,
                ZAPPING_NESW,
                FISH,
                WALKING_E_WITH_FISH,
                SPRING,
                LEAF,
                ARROWS,
                SNAIL,
                LIGHTNING,
                STARS,
                PLATE,
                PAUSE_MENU_CONTINUE,
                PAUSE_MENU_QUIT,
                SETTINGS_BG,
                IMAGE_CONTROL,
                IMAGE_SOUND,
                IMAGE_SIZE,
                IMAGE_OK,
                IMAGE_CANCEL,
                IMAGE_TICK,
                IMAGE_CONTROL_FAINT,
                IMAGE_SOUND_FAINT,
                IMAGE_SIZE_FAINT,
                IMAGE_OK_FAINT,
                IMAGE_CANCEL_FAINT,
                IMAGE_TICK_FAINT,
                INTRO_SCREEN,
                MESSAGE_BOARD,
        }; // enum Image_Name

        static const int NUM_FONTS = 6;
        enum Font_Name { 
                SMALL_FONT, 
                SMALL_FONT_FAINT, 
                LARGE_FONT, 
                LARGE_FONT_FAINT, 
                WHITE_FONT,
                TUTORIAL_FONT,  // DejaVu Sans Mono black 20pt
        }; // enum Font_Name
        
    private:
    
        SDL_Surface* my_images[NUM_IMAGES];
        SDL_Surface* my_font_images[NUM_FONTS];
        SFont my_fonts[NUM_FONTS];
        
        
    public:
    
        GfxStore();
        ~GfxStore();
        
        SDL_Surface* get_image(Image_Name n);
        const SFont& get_font(Font_Name n) const;
        
}; // class GfxStore    
        
} // namespace FS




#endif
