// $Id: log.cc,v 1.8 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "log.h"
#include <iostream>




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::Log::Log( int _x, int _y, int _width, int _my_row, LogColor color )
{
    x = old_x = _x;
    y = old_y = _y;
    width = _width;
    height = CELL_HEIGHT;
    
    my_row = _my_row;
    original_color = current_color = color;
    
    sprite_src_rect.x = 0;
    sprite_src_rect.y = 0;
    sprite_src_rect.w = width;
    sprite_src_rect.h = height;
    
    sprite_dest_rect.y = y;
    
    erase_rect.y = y;
    erase_rect.h = height;
    
    update_rect.y = y;
    update_rect.h = height;
    
    coll_box_x_offset = 0;
    coll_box_y_offset = 0;
    coll_box.w = width;
    coll_box.h = height;
    
    end_src_rect.x = 242;
    end_src_rect.y = 0;
    end_src_rect.w = 8;
    end_src_rect.h = 50;
    
    end_dest_rect.y = y;
    
    virtual_x = 0;
    virtual_old_x = 0;
    
} // FS::Log::Log
    
// ******************
// *** DESTRUCTOR ***
// ******************
FS::Log::~Log()
{
    // NYI
    
} // FS::Log::~Log()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

void FS::Log::update( const ScrollingData & sd )
{
    scroll_x(sd);   // see MapSprite::scroll_x
    
    if ( calc_rects(sd) )
    {
        if ( erase_rect_visible )
        {
            my_blitter->request_erase( GfxStore::BACKGROUND, erase_rect );
        } // if
        
        if ( sprite_visible )
        {
            my_blitter->request_blit_image( 
                    (GfxStore::Image_Name) (current_color + 6), // FIXME!!!
                    sprite_src_rect, sprite_dest_rect );
        } // if
        
        if ( end_visible )
        {
            my_blitter->request_blit_image(
                    (GfxStore::Image_Name) (current_color + 6), // FIXME!!!
                    end_src_rect, end_dest_rect );
        } // if
        
        my_blitter->request_update( update_rect );
    } // if
    
    // Albert needs to know these for when he's travelling on a log.
    num_pixels_moved = sd.dxs[my_row];
    my_direction = sd.current_directions[my_row];
    
} // FS::Log::update
    
// **************************************************
    
bool FS::Log::calc_rects( const ScrollingData & sd )
{
    // For the purposes of the following code, all x-coords should
    // be < SCREEN_WIDTH. For want of a better name, we'll call these
    // values virtual_x and virtual_old_x, because they're the 'onscreen'
    // values as used by SDL.
    
    if ( x >= SCREEN_WIDTH )
    {
        virtual_x = -(sd.row_lengths_in_pixels[my_row] - x);
    }
    else
    {
        virtual_x = x;
    } // if ... else
    
    if ( old_x >= SCREEN_WIDTH )
    {
        virtual_old_x = -(sd.row_lengths_in_pixels[my_row] - old_x);
    }
    else
    {
        virtual_old_x = old_x;
    } // if ... else
    
    sprite_dest_rect.x = virtual_x;
    
    switch ( sd.current_directions[my_row] )
    {
        case LEFT_TO_RIGHT:
            erase_rect.x = virtual_old_x;
            break;
        case RIGHT_TO_LEFT:
            erase_rect.x = virtual_x;
            break;
    } // switch
    erase_rect.w = width + sd.dxs[my_row];
    
    end_dest_rect.x = virtual_x + width - end_src_rect.w;
    
    sprite_visible = sprite_dest_rect.x + width >= 0;
    erase_rect_visible = erase_rect.x + erase_rect.w >= 0;
    end_visible = end_dest_rect.x + end_src_rect.w >= 0;
    
    if ( sprite_visible || erase_rect_visible )
    {
        switch ( sd.current_directions[my_row] )
        {
            case LEFT_TO_RIGHT:
                update_rect.x = erase_rect.x;
                update_rect.w = sprite_dest_rect.x + width - erase_rect.x;
                break;
            case RIGHT_TO_LEFT:
                update_rect.x = sprite_dest_rect.x;
                update_rect.w = erase_rect.x + erase_rect.w - sprite_dest_rect.x;
                break;
        } // switch
        clip_update_rect_x(SCREEN_WIDTH);
        return true;          
    }
    else
    {
        return false;
    } // if ... else
    
} // FS::Log::calc_rects

// **************************************************

// Override Sprite::get_coll_box so we can use 'virtual' x-coord for 
// calculating collision box. Otherwise, for logs moving r-to-l, it looks
// as though the log goes entirely offscreen as soon as x < 0, because then
// x is recorded as a +ve value at far right of row length in pixels.(?!)
// N.B. virtual_x is updated on every call to Log::update, so we can be sure
// it is always valid.
const SDL_Rect & FS::Log::get_coll_box()
{
    coll_box.x = virtual_x + coll_box_x_offset;
    coll_box.y = y + coll_box_y_offset;
    
    return coll_box;    
    
} // FS::Log::get_coll_box

