// $Id: log.h,v 1.8 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#ifndef _LOG_H_
#define _LOG_H_




#include "mapsprite.h"
#include "enums.h"
#include "gfxstore.hh"
#include "blitter.hh"




extern FS::GfxStore * my_gfx;
extern FS::Blitter * my_blitter;




namespace FS
{
    
class Log : public MapSprite
{
    private:
            
        LogColor original_color;
        LogColor current_color;
        
        // If the log is not full size (i.e. 250 pixels in width) then
        // the end bit will be cut off at the right-hand side. end_rect_src 
        // is the rectangle for that piece and end_rect_dest is where it
        // should be copied/blitted to.
        SDL_Rect end_src_rect;
        SDL_Rect end_dest_rect;
        bool end_visible;
        
        // Used by calc_rects to hold -ve 'versions' of log's x-coord if
        // it's >= SCREEN_WIDTH. virtual_x is also used in Log::get_coll_box
        // when log's moving r-to-l to ensure the SDL_Rect is valid.
        int virtual_x;
        int virtual_old_x;
    
        // Store these when Log is updated, because they're needed by
        // Albert if he happens to be floating on this log.
        int num_pixels_moved;
        Direction my_direction;
        
        // Calculates values for sprite_dest_rect, erase_rect, end_dest_rect
        // and update_rect (if appropriate). Also initializes sprite_visible,
        // erase_visible and end_visible. Returns true if sprite needs
        // erasing/redrawing; else false.
        bool calc_rects( const ScrollingData & sd );
        
    public:
            
        Log( int _x, int _y, int _width, int _my_row, LogColor color );
        ~Log();
        
        LogColor get_color() const { return current_color; }
        void set_color(LogColor c) { current_color = c; }
        
        int get_num_pixels_moved() const { return num_pixels_moved; }
        Direction get_direction() const { return my_direction; }
        
        // Provide an implementation of pure virtual function MapSprite::update
        void update( const ScrollingData & sd );
        
        void reset() { current_color = original_color; }
        
        // Override Sprite::get_coll_box.
        const SDL_Rect & get_coll_box();
            
}; // class Log

} // namespace FS




#endif
    
