// $Id: pausemenu.cc,v 1.7 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "pausemenu.h"




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::PauseMenu::PauseMenu()
{
    menu_update_rect.x = 295;
    menu_update_rect.y = 236;
    menu_update_rect.w = 210;
    menu_update_rect.h = 127;
    
} // FS::PauseMenu::PauseMenu

// ******************
// *** DESTRUCTOR ***
// ******************
FS::PauseMenu::~PauseMenu()
{
    // ...
    
} // FS::PauseMenu::~PauseMenu




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::PauseMenu::reset()
{
    current_selection = PMS_CONTINUE;    // this is the default
    
} // FS::PauseMenu::reset

// **************************************************

FS::PauseMenu::PauseMenuSelection FS::PauseMenu::update()
{
    bool selection_made = false;
    UserInput::Keyboard_Event ke;
    
    while ( my_input->next_event(ke) )
    {
        if (ke == UserInput::ESCAPE_PRESSED)
        {
            current_selection = PMS_CONTINUE;
            selection_made = true;
            break;
        }
        else if (ke == UserInput::JUMP_KEY_PRESSED)
        {
            selection_made = true;
            break;
        }
        else if (ke == UserInput::UP_KEY_PRESSED || ke == UserInput::DOWN_KEY_PRESSED)
        {
            current_selection = ((current_selection == PMS_CONTINUE) ?
                    PMS_QUIT : PMS_CONTINUE);
        } // if ... else
    } // while
    
    if (selection_made)
    {
        return current_selection;
    }
    else
    {
        return PMS_NONE;
    } // if ... else
  
} // FS::PauseMenu::update

// **************************************************
        
void FS::PauseMenu::draw()
{
    GfxStore::Image_Name img;
    if ( current_selection == PMS_CONTINUE )
    {
        img = GfxStore::PAUSE_MENU_CONTINUE;
    }
    else
    {
        img = GfxStore::PAUSE_MENU_QUIT;
    } // if ... else
    
    my_blitter->request_erase( GfxStore::BACKGROUND, menu_update_rect );
    my_blitter->request_blit_image( img, menu_update_rect );
    my_blitter->request_update( menu_update_rect );
    
} // FS::PauseMenu::draw

// **************************************************

void FS::PauseMenu::erase()
{
    my_blitter->request_erase( GfxStore::BACKGROUND, menu_update_rect );
    my_blitter->request_update( menu_update_rect );
    
} // FS::PauseMenu::erase

// **************************************************

bool FS::PauseMenu::has_pressed_jump() const
{
    bool jump_pressed = false;
    UserInput::Keyboard_Event ke;

    while ( my_input->next_event(ke) )
    {
        if ( ke == UserInput::JUMP_KEY_PRESSED )
        {
            jump_pressed = true;
        } // if
    } // while
    
    return jump_pressed;
    
} // FS::PauseMenu::has_pressed_jump
