// $Id: playerdata.cc,v 1.12 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "playerdata.h"




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::PlayerData::PlayerData()
{
    erase_rect.y = CRYSTALS_HELD_Y;
    erase_rect.w = CRYSTAL_WIDTH;
    erase_rect.h = CRYSTAL_HEIGHT;
    
    dest_rect.y = CRYSTALS_HELD_Y;
    
    dirty_rect.x = CRYSTALS_HELD_X_OFFSET;
    dirty_rect.y = CRYSTALS_HELD_Y;        
    dirty_rect.h = CRYSTAL_HEIGHT;
    
    score_update_rect.x = 191;
    score_update_rect.y = 9;
    score_update_rect.w = 70;
    score_update_rect.h = 15;
    
    score_dest_rect.x = 190;
    score_dest_rect.y = 4;
    
    lives_update_rect.x = 41;
    lives_update_rect.y = 9;
    lives_update_rect.w = 10;
    lives_update_rect.h = 15;
    
    lives_dest_rect.x = 40;
    lives_dest_rect.y = 4;
    
    level_update_rect.x = 707;
    level_update_rect.y = 9;
    level_update_rect.w = 22;
    level_update_rect.h = 15;
    
    level_dest_rect.x = 706;
    level_dest_rect.y = 4;
    
} // FS::PlayerData::PlayerData()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::PlayerData::~PlayerData() {}




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************
        
void FS::PlayerData::reset(int start_level)
{
    level = start_level;  // levels start at ONE!!!
    num_lives = STARTING_NUM_LIVES;
    score = 0;
    
} // FS::PlayerData::reset()
     
// **************************************************

void FS::PlayerData::set_max_crystals(int mc)
{
    max_crystals = mc;
    
    // This only needs to be set once at the start of each new level.     
    dirty_rect.w = (max_crystals * CRYSTAL_WIDTH) + 
            ((max_crystals - 1) * CRYSTALS_HELD_X_GAP);
    
} // FS::PlayerData::set_max_crystals()
        
// **************************************************

bool FS::PlayerData::is_allowed_on_log(LogColor lc)
{
    bool is_ok = false;
            
    switch (lc)
    {
        case L_RED:
        case L_YELLOW:
        case L_BLUE:
            crystals_iter = find(crystals.begin(), crystals.end(), (CrystalColor) lc);
            if (crystals_iter != crystals.end())
            {
                crystals.erase(crystals_iter);
                is_ok = true;
                break;
            } // if
            break;
            
        case L_ORANGE:
            is_ok = check_crystals(C_RED, C_YELLOW);
            break;
            
        case L_GREEN:
            is_ok = check_crystals(C_BLUE, C_YELLOW);
            break;
            
        case L_PURPLE:
            is_ok = check_crystals(C_BLUE, C_RED);
            break;
            
        case L_BROWN:
            is_ok = true;
            break;
            
        // Should never happen - just to avoid warnings from compiler...
        case L_BACKGROUND:
            break;
            
    } // switch

    // If we haven't got an exact colour-match, is there a white
    // crystal to save the day?
    if (!is_ok)
    {
        crystals_iter = find(crystals.begin(), crystals.end(), C_WHITE);
        if (crystals_iter != crystals.end())
        {
            crystals.erase(crystals_iter);
            is_ok = true;
        } // if
    } // if
            
    return is_ok;
    
} // FS::PlayerData::is_allowed_on_log()
    
// **************************************************

bool FS::PlayerData::check_crystals(CrystalColor color_0, CrystalColor color_1)
{
    // We need TWO iterators for this function.
    std::list<CrystalColor>::iterator iter_0;
    std::list<CrystalColor>::iterator iter_1;
    
    iter_0 = find(crystals.begin(), crystals.end(), color_0);
    iter_1 = find(crystals.begin(), crystals.end(), color_1);
    
    if ( iter_0 != crystals.end() && iter_1 != crystals.end() )
    {
        crystals.erase(iter_0);
        crystals.erase(iter_1);
        return true;
    } 
    else
    {
        return false;
    } // if ... else
    
} // FS::PlayerData::check_crystals()
    
// **************************************************

FS::CrystalColor FS::PlayerData::add_crystal(CrystalColor cc)
{
    // Simple case - see if we've already got a spare 'slot'.
    if ( crystals.size() < max_crystals )
    {
        crystals.push_back(cc);
        return C_NONE;  // nothing swapped
    } // if
    
    // No spare slot, so pop front (and return) and push new 
    // crystal to back.
    CrystalColor removed = crystals.front();
    crystals.pop_front();
    crystals.push_back(cc);
    return removed;
    
} // FS::PlayerData::add_crystal() 

// **************************************************

// FIXME: a bit of hackery here (separate index, inefficient erasing).
void FS::PlayerData::draw_crystals_held()
{
    int i = 0;
    crystals_iter = crystals.begin();
    while ( crystals_iter != crystals.end() )
    {
        dest_rect.x = CRYSTALS_HELD_X_OFFSET +
                (i * CRYSTAL_WIDTH) + (i * CRYSTALS_HELD_X_GAP);
        my_blitter->request_blit_image(
                (GfxStore::Image_Name) (*crystals_iter + 13), 
                dest_rect);
        ++i;
        ++crystals_iter;
    } // while
    
    my_blitter->request_erase(GfxStore::BACKGROUND, dirty_rect);
    my_blitter->request_update(dirty_rect);
    
} // FS::PlayerData::draw_crystals_held()

// **************************************************

void FS::PlayerData::set_crystals_held(CrystalColor * ccs, int n)
{
    crystals.clear();
    for (int i = 0; i < n; ++i)
    {
        crystals.push_back(ccs[i]);
    } // for

} // FS::PlayerData::set_crystals_held()

// **************************************************

// Returns true if there's another level to play, otherwise false.
bool FS::PlayerData::inc_level()
{
    ++level;
    return true;
    
} // FS::PlayerData::inc_level()

// **************************************************

void FS::PlayerData::draw_score()
{
    sprintf( score_str, "%06d", score );
    my_blitter->request_erase( GfxStore::BACKGROUND, score_update_rect );
    my_blitter->request_blit_text( GfxStore::WHITE_FONT, score_dest_rect, score_str );
    my_blitter->request_update( score_update_rect );
    
} // FS::PlayerData::draw_score
    
// **************************************************

void FS::PlayerData::draw_lives()
{
    sprintf( lives_str, "%d", num_lives );
    my_blitter->request_erase( GfxStore::BACKGROUND, lives_update_rect );
    my_blitter->request_blit_text( GfxStore::WHITE_FONT, lives_dest_rect, lives_str );
    my_blitter->request_update( lives_update_rect );
    
} // FS::PlayerData::draw_lives

// **************************************************

void FS::PlayerData::draw_level()
{
    sprintf( level_str, "%d", level );
    my_blitter->request_erase( GfxStore::BACKGROUND, level_update_rect );
    my_blitter->request_blit_text( GfxStore::WHITE_FONT, level_dest_rect, level_str );
    my_blitter->request_update( level_update_rect );
    
} // FS::PlayerData::draw_level

// **************************************************

bool FS::PlayerData::check_crystals(const CrystalColor* ccs, int n)
{
    // Simple case - we don't need any crystals!
    if ( n == 0 ) { return true; }
    
    // First, need a copy of 'crystals' list.
    std::list<CrystalColor> crystals_copy(crystals.begin(), crystals.end());
    
    bool found_all = true;
    for (int i = 0; i < n; ++i)
    {
        crystals_iter = find(crystals_copy.begin(), crystals_copy.end(), ccs[i]);
        if ( crystals_iter == crystals_copy.end() ) 
        {
            // didn't find crystal - so exit loop and return false
            found_all = false;
            break;
        }
        else
        {
            // found crystal, so remove it from list ready for next search
            crystals_copy.erase(crystals_iter);
        } // if ... else
    } // for
    
    return found_all;
    
} // FS::PlayerData::check_crystals

// **************************************************

void FS::PlayerData::print_crystals_held() const
{
    std::cout << "Crystals held: ";
    
    std::list<CrystalColor>::const_iterator c_iter = crystals.begin();
    while (c_iter != crystals.end())
    {
        std::cout << *c_iter << ",";
        ++c_iter;
    } // while
    std::cout << std::endl;
    
} // FS::PlayerData::print_crystals_held

    
    






 
