// $Id: playerdata.h,v 1.12 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#ifndef _PLAYERDATA_H_
#define _PLAYERDATA_H_




#include "enums.h"
#include "constants.h"
#include "SDL.h"
#include <iostream>
#include <cstdlib>
#include <cstdio>
#include "gfxstore.hh"
#include "blitter.hh"
#include <list>




extern FS::GfxStore * my_gfx;
extern FS::Blitter * my_blitter;




namespace FS
{
    
class PlayerData
{
    private:
            
        int level;
        int num_lives;
        int score;
        
        std::list<CrystalColor> crystals;
        std::list<CrystalColor>::iterator crystals_iter;
        unsigned int max_crystals;
        
        bool check_crystals(CrystalColor color_0, CrystalColor color_1);
        
        SDL_Rect dest_rect;
        SDL_Rect erase_rect;
        SDL_Rect dirty_rect;
        
        char score_str[7];
        SDL_Rect score_update_rect;
        SDL_Rect score_dest_rect;
        char lives_str[2];
        SDL_Rect lives_update_rect;
        SDL_Rect lives_dest_rect;
        char level_str[3];
        SDL_Rect level_update_rect;
        SDL_Rect level_dest_rect;
        
        void print_crystals_held() const;
        
    public:
            
        PlayerData();
        ~PlayerData();
        
        void reset(int start_level);
        void set_max_crystals(int mc);
        void set_crystals_held(CrystalColor * ccs, int n);
        
        void draw_crystals_held();
        
        // NEW!
        bool is_allowed_on_log(LogColor lc);
        CrystalColor add_crystal(CrystalColor cc);
        
        bool inc_level();
        int get_level() const { return level; }
        
        void inc_score(int pts) { score += pts; }
        int get_score() const { return score; }
        
        // returns number of lives left
        void dec_num_lives() { --num_lives; }
        int get_num_lives() const { return num_lives; }
        
        const SDL_Rect & get_crystals_held_bounding_box() { return dirty_rect; }
        
        void draw_score();
        void draw_lives();
        void draw_level();
        
        bool check_crystals(const CrystalColor* ccs, int n);
        
}; // class PlayerData

} // namespace FS




#endif
        
