// $Id: playmodel.h,v 1.14 2009-01-25 09:29:46 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#ifndef _PLAY_MODEL_H_
#define _PLAY_MODEL_H_




#include "spritemanager.h"
#include "playerdata.h"
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include "enums.h"
#include "soundengine.h"
#include "constants.h"
#include <string>
#include "scroller.hh"
#include "gametimer.h"
#include "pausemenu.h"
#include "tutorial.hh"




extern bool test_level;
extern char * test_file;
// This bool allows Albert and PlayModel to communicate with each other.
extern bool escape_pressed;
extern FS::Mode current_mode;




namespace FS
{

    
class PlayModel
{
    public:
    
        // PAUSE_AUTO only in MODE_TUTORIAL - game has been automatically
        // paused by program.        
        //enum PauseType { PAUSE_NONE, PAUSE_USER, PAUSE_AUTO };
    
    private:
            
        SpriteManager my_sprite_mngr;
        PlayerData my_player_data;
        Scroller my_scroller;
        PauseMenu my_pause_menu;
        GameTimer * timer_ptr;
        Tutorial my_tutorial;
        SoundEngine* snd_engine_ptr;
        
        int start_time;
        
        void load_level(int lev);    
        
        int num_starting_crystals;
        // FIXME: need a symbolic constant!
        CrystalColor starting_crystals[5];
        
        int current_update_time;
        int previous_update_time;
        
        void handle_escape_pressed( PlayReport report );
        
        void check_task();
        //void display_task_msg(const Tutorial::Task* task);
        bool have_drawn_reminder;
        
        PauseType my_pause;
        
    public:
        
        PlayModel( GameTimer * _timer_ptr );
        ~PlayModel();
        
        void init_sprite_manager(SoundEngine * se);
        
        void new_game(int start_level = 1);
        void new_level(int st);
                
        PlayReport update(int gt);
        
        int get_level() const { return my_player_data.get_level(); }
        int get_score() const { return my_player_data.get_score(); }
        
}; // class PlayModel

} // namespace FS




#endif

