// $Id: settingsdisplay.cc,v 1.12 2009-01-25 09:29:47 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "settingsdisplay.h"




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::SettingsDisplay::SettingsDisplay(Settings * _my_settings)
{
    my_settings = _my_settings;
    init_rects();
    
} // FS::SettingsDisplay::SettingsDisplay()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::SettingsDisplay::~SettingsDisplay()
{
    // nothing
            
} // FS::SettingsDisplay::~SettingsDisplay()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::SettingsDisplay::start_draw()
{
    current_option = previous_option = CONTROL;
    
    init_scratch();
    
    for (int i = 0; i < NUM_OPTIONS; ++i)
    {
        if ( i == current_option )
        {
            my_blitter->request_blit_image( ((GfxStore::Image_Name) (i + BOLD_OFFSET)),
                    option_rects[i] );
            draw_tick(i, true);
        }
        else
        {
            my_blitter->request_blit_image( ((GfxStore::Image_Name) (i + FAINT_OFFSET)),
                    option_rects[i] );
            draw_tick(i, false);
        } // if ... else
    } // for
    
    my_blitter->blit_and_update( GfxStore::SETTINGS_BG );
    
} // FS::SettingsDisplay::start_draw()

// **************************************************

bool FS::SettingsDisplay::redraw()
{
    // FIXME: make sure event list is empty if returning true.
    
    bool changed = false;
    bool quit = false;  // return this value
    
    previous_option = current_option;
    
    UserInput::Keyboard_Event ke;
    while ( my_input->next_event(ke) )
    {
        if ( ke == UserInput::ESCAPE_PRESSED )  
        {
            quit = true;
            break;  // 'short-circuit'
        }
        else if ( ke == UserInput::UP_KEY_PRESSED )
        {
            --current_option;
            if (current_option < 0) 
            {
                current_option = NUM_OPTIONS - 1;
            } // if
            changed = true;
        }
        else if ( ke == UserInput::DOWN_KEY_PRESSED )
        {
            current_option = (current_option + 1) % NUM_OPTIONS;
            changed = true;
        }
        else if ( ke == UserInput::JUMP_KEY_PRESSED )
        {
            if (current_option == OK)
            {
                write_scratch();
                quit = true;
                break;
            }
            else if (current_option == CANCEL)
            {
                quit = true;
                break;
            }
            else
            {
                swap_scratch(current_option);
                changed = true;
            } // if ... else
        } // if ... else
    } // while
    
    if (changed)
    {
        do_redraw();
    } // if
    
    return quit;
    
} // FS::SettingsDisplay::redraw()

// **************************************************

void FS::SettingsDisplay::init_rects()
{
    int xs[] = { 80, 80, 80, 233, 215 };
    int ys[] = { 120, 240, 360, 470, 510 };
    int ws[] = { 340, 340, 340, 33, 69 };
    int hs[] = { 86, 86, 86, 17, 18 };
    
    for (int i = 0; i < NUM_OPTIONS; ++i)
    {
        option_rects[i].x = xs[i];
        option_rects[i].y = ys[i];
        option_rects[i].w = ws[i];
        option_rects[i].h = hs[i];
    } // for
    
} // FS::SettingsDisplay::init_rects()

// **************************************************

void FS::SettingsDisplay::do_redraw()
{
    // erase last option and redraw faint
    my_blitter->request_erase( GfxStore::SETTINGS_BG, option_rects[previous_option] );
    my_blitter->request_blit_image( ((GfxStore::Image_Name) (previous_option + FAINT_OFFSET)),
            option_rects[previous_option] );
    draw_tick(previous_option, false);
    
    // redraw current option normal (i.e. highlighted)
    // erase first in case user has pressed 'jump' (and we're swapping ticks)
    my_blitter->request_erase( GfxStore::SETTINGS_BG, option_rects[current_option] );
    my_blitter->request_blit_image( ((GfxStore::Image_Name) (current_option + BOLD_OFFSET)),
            option_rects[current_option] );
    draw_tick(current_option, true);
    
    my_blitter->request_update( option_rects[previous_option] );
    my_blitter->request_update( option_rects[current_option] );
    
    my_blitter->blit_and_update();
    
} // FS::SettingsDisplay::do_redraw()

// **************************************************

void FS::SettingsDisplay::init_scratch()
{
    scratch_joystick_selected = my_settings->is_joystick_selected();
    scratch_sound = my_settings->is_sound();
    scratch_fullscreen = my_settings->is_fullscreen();
    
} // FS::SettingsDisplay::init_scratch()

// **************************************************

void FS::SettingsDisplay::draw_tick(int option, bool highlighted)
{
    if (option > SIZE)
    {
        return;
    } // if
    
    SDL_Rect tick_rect;
    tick_rect.y = option_rects[option].y + TICK_Y_OFFSET;
    tick_rect.w = 50;
    tick_rect.h = 50;
    
    if (option == CONTROL)
    {
        if (scratch_joystick_selected)
        {
            tick_rect.x = option_rects[option].x + RIGHT_TICK_X_OFFSET;
        }
        else 
        {
            tick_rect.x = option_rects[option].x + LEFT_TICK_X_OFFSET;
        } // if ... else
    }
    else if (option == SOUND)
    {
        if (scratch_sound)
        {
            tick_rect.x = option_rects[option].x + LEFT_TICK_X_OFFSET;
        }
        else
        {
            tick_rect.x = option_rects[option].x + RIGHT_TICK_X_OFFSET;
        } // if ... else
    }
    else if (option == SIZE)
    {
        if (scratch_fullscreen)
        {
            tick_rect.x = option_rects[option].x + RIGHT_TICK_X_OFFSET;
        }
        else 
        {
            tick_rect.x = option_rects[option].x + LEFT_TICK_X_OFFSET;
        } // if ... else
    } // if ... else
    
    my_blitter->request_blit_image( 
            ((highlighted) ? GfxStore::IMAGE_TICK : GfxStore::IMAGE_TICK_FAINT),
            tick_rect );
    
} // FS::SettingsDisplay::draw_tick()
       
// **************************************************

void FS::SettingsDisplay::swap_scratch(int option)
{
    switch (option)
    {
        case CONTROL:
            scratch_joystick_selected = !scratch_joystick_selected;
            break;
        case SOUND:
            scratch_sound = !scratch_sound;
            break;
        case SIZE:
            scratch_fullscreen = !scratch_fullscreen;
            break;
        case OK:
        case CANCEL:
            break;
    } // switch
    
} // FS::SettingsDisplay::swap_scratch()

// **************************************************

void FS::SettingsDisplay::write_scratch()
{
    my_settings->set_joystick_selected(scratch_joystick_selected);
    my_settings->set_sound(scratch_sound);
    my_settings->set_fullscreen(scratch_fullscreen);
    
} // FS::SettingsDisplay::write_scratch()

// **************************************************
