// $Id: settingsdisplay.h,v 1.11 2009-01-25 09:29:47 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#ifndef _SETTINGSDISPLAY_H_
#define _SETTINGSDISPLAY_H_




#include "settings.h"
#include "SDL.h"
#include "userinput.hh"
#include "gfxstore.hh"
#include "blitter.hh"




extern FS::UserInput * my_input;
extern FS::GfxStore * my_gfx;
//extern SDL_Surface * screen;
extern FS::Blitter * my_blitter;




namespace FS
{
    
class SettingsDisplay
{

    private:
        
        static const int NUM_OPTIONS = 5;
        enum { CONTROL, SOUND, SIZE, OK, CANCEL };
        
        static const int LEFT_TICK_X_OFFSET = 110;  // left column
        static const int RIGHT_TICK_X_OFFSET = 180;  // right column
        static const int TICK_Y_OFFSET = 36;
        
        static const int BOLD_OFFSET = GfxStore::IMAGE_CONTROL;
        static const int FAINT_OFFSET = GfxStore::IMAGE_CONTROL_FAINT;
        
        SDL_Rect option_rects[NUM_OPTIONS];
        
        Settings * my_settings;
    
        int current_option;
        int previous_option;
        
        void init_rects();
        void do_redraw();
        void draw_tick(int option, bool highlighted);
        
        bool scratch_joystick_selected;
        bool scratch_sound;
        bool scratch_fullscreen;
        void init_scratch();
        void write_scratch();
        void swap_scratch(int option);
            
    public:
        
        SettingsDisplay(Settings * _my_settings);
        ~SettingsDisplay();
        
        void start_draw();
        
        // Returns true if user has selected 'OK' or 'Cancel' or 
        // pressed 'ESC' - all processing finished.
        bool redraw();
        
}; // class SettingsDisplay

} // namespace FS




#endif
            
