// $Id: spritemanager.h,v 1.13 2009-01-25 09:29:47 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#ifndef _SPRITEMANAGER_H_
#define _SPRITEMANAGER_H_




#include "enums.h"
#include "SDL.h"
#include "log.h"
#include "crystal.h"
#include "constants.h"
#include "albert.h"
#include "playerdata.h"
#include "fish.h"
#include "soundengine.h"
#include "bonus.h"
#include <vector>
#include "collectablesprite.h"
#include "collisions.h"
#include <fstream>
#include <cstdio>
#include <iostream>
#include <cstdlib>
#include "userinput.hh"
#include "gfxstore.hh"
#include "blitter.hh"
#include "scroller.hh"
#include "pausemenu.h"




extern FS::UserInput * my_input;
extern FS::GfxStore * my_gfx;
extern FS::Blitter * my_blitter;




namespace FS
{
    
class SpriteManager
{    
    public:
            
        enum State 
        { 
            PLAYING_GAME,
            REQUEST_LEVEL_RESTART, 
            RUNNING_LEVEL_COMPLETED_ROUTINE,
            LEVEL_COMPLETED,
            RUNNING_GAME_OVER_ROUTINE,
            GAME_OVER
        }; // enum SpriteMngrState
        
        enum CatLogCollisionType { ALLOWED, DISALLOWED, MISSED }; 
        
    private:
    
        PlayerData * player_data_ptr;    // from PlayModel
        SoundEngine * sound_engine_ptr;    // from GameManager
        Scroller * scroller_ptr;
        PauseMenu * pause_menu_ptr;
        
        bool pause_menu_needs_erasing;
        bool pause_menu_needs_drawing; 
        
        Albert the_cat;
        Fish the_fish;
        std::vector<Log *> my_logs;
        std::vector<Log *>::iterator logs_iter;
        std::vector<CollectableSprite *> my_collectables;
        std::vector<CollectableSprite *>::iterator collectables_iter;
        
        State current_state;
        
        int time_passed;
        bool is_first_pass;
        
        bool redraw_crystals;
        bool redraw_fish;
        bool redraw_score;
        bool redraw_level;
        bool redraw_lives;
        
        // Use these to help with collision detection.
        int rows_min_ys[NUM_ROWS];
        int rows_max_ys[NUM_ROWS];
        
        void create_logs();
        void create_collectables();
        void delete_logs_and_collectables();
        
        void check_collisions();
        
        RowDescription calc_alberts_row();
        
        CatLogCollisionType check_cat_log_collision(int row);
        void check_cat_collectable_collision(int row);
        void handle_crystal_collision(Crystal * c);
        void handle_bonus_collision(Bonus * b);
        void set_logs_to_brown(int row);
        
        // These two variables work together to prevent Albert
        // from picking up a swapped crystal over and over...
        bool cat_crystal_collisions_disabled;
        Crystal* disabled_crystal;
        
        RowDescription alberts_row;
        
    public:
    
        SpriteManager();
        ~SpriteManager();
        
        void set_player_data( PlayerData* pd ) { player_data_ptr = pd; }
        void set_sound_engine( SoundEngine* se ) { sound_engine_ptr = se; }
        void set_scroller( Scroller* s ) { scroller_ptr = s; }
        void set_pause_menu( PauseMenu* pm ) { pause_menu_ptr = pm; }
        void create_scrolling_sprites();
        
        void reset();
        void restart_level();
        
        State update( int t, const ScrollingData& sd, PauseType pause_t );
       
        void activate_pause_menu();
        void deactivate_pause_menu();
        
        int get_cats_row() const { return the_cat.get_host_log_row(); }
        const Log* get_cats_host_log() const { return the_cat.get_host_log(); }
        
        void redraw_screen();
        
        RowDescription get_alberts_row() const { return alberts_row; }
        const LogColor get_host_log_color() const;
        
}; // class SpriteManager

} // namespace FS




#endif

        
    
