// $Id: titlescreendisplay.cc,v 1.15 2009-01-25 09:29:47 matthew Exp $

// Fish Supper
// Copyright 2006, 2007, 2009 Matthew Clarke <matthew_larcke@lycos.co.uk>

// This file is part of Fish Supper.

// Fish Supper is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Fish Supper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Fish Supper.  If not, see <http://www.gnu.org/licenses/>.




#include "titlescreendisplay.h"




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::TitleScreenDisplay::TitleScreenDisplay()
{
    int temp1[] = { 180, 250, 320, 390, 460 };
    std::string temp2[] = { "Play", "Tutorial", "Settings", "High Scores", "Quit" };
    for (int i = 0; i < NUM_SELECTIONS; ++i)
    {
        selection_names[i] = temp2[i];
        selection_rects[i].x = SELECTION_X;
        selection_rects[i].y = temp1[i];
        selection_rects[i].w = my_gfx->get_font(GfxStore::LARGE_FONT).getTextWidth(temp2[i]);
        selection_rects[i].h = my_gfx->get_font(GfxStore::LARGE_FONT).getHeight();
    } // for
        
    current_selection = 0;
  
} // FS::TitleScreenDisplay::TitleScreenDisplay()    
 
// ******************
// *** DESTRUCTOR ***
// ******************
FS::TitleScreenDisplay::~TitleScreenDisplay()
{
    // nothing
    
} // FS::TitleScreenDisplay::~TitleScreenDisplay()    




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::TitleScreenDisplay::start_draw()
{
    for (int i = 0; i < NUM_SELECTIONS; ++i)
    {   
        if (i == current_selection)
        {
            my_blitter->request_blit_text( GfxStore::LARGE_FONT, selection_rects[i],
                    selection_names[i].c_str() );
        }
        else 
        {
            my_blitter->request_blit_text( GfxStore::LARGE_FONT_FAINT, selection_rects[i],
                    selection_names[i].c_str() );
        } // if ... else
    } // for    
           
    my_blitter->blit_and_update( GfxStore::INTRO_SCREEN );
    
} // FS::TitleScreenDisplay::start_draw()

// **************************************************

FS::TitleScreenDisplay::Selection FS::TitleScreenDisplay::update()
{
    bool changed = false;
    Selection sel = NONE;
    UserInput::Keyboard_Event ke;
    old_selection = current_selection;
    
    while ( my_input->next_event(ke) )
    {
        if ( ke == UserInput::ESCAPE_PRESSED )
        {
            sel = QUIT;
            break;
        } 
        else if ( ke == UserInput::JUMP_KEY_PRESSED )
        {
            sel = (Selection) current_selection;
            break;
        }
        else if ( ke == UserInput::DOWN_KEY_PRESSED )
        {
            current_selection = (current_selection + 1) % NUM_SELECTIONS;
            changed = true;
        }
        else if ( ke == UserInput::UP_KEY_PRESSED )
        {
            --current_selection;
            if (current_selection < 0)
            {
                current_selection = (NUM_SELECTIONS - 1);
            } // if
            changed = true;
        } // if ... else
    } // while
    
    if (changed)
    {
        redraw_selection();
    } // if
    
    return sel;
            
} // FS::TitleScreenDisplay::update() 

// **************************************************

void FS::TitleScreenDisplay::redraw_selection()
{    
    my_blitter->request_erase( GfxStore::INTRO_SCREEN, selection_rects[old_selection] );
    my_blitter->request_blit_text( GfxStore::LARGE_FONT_FAINT, selection_rects[old_selection],
            selection_names[old_selection].c_str() );
    my_blitter->request_update( selection_rects[old_selection] );
    
    my_blitter->request_blit_text( GfxStore::LARGE_FONT, selection_rects[current_selection],
            selection_names[current_selection].c_str() );
    my_blitter->request_update( selection_rects[current_selection] );
    
    my_blitter->blit_and_update();
          
} // FS::TitleScreenDisplay::redraw_selection()                
    
// **************************************************
